/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: BSP tree header file                                       */
/*        Version: 0.3 (010327)                                               */
/*                                                                            */
/*        Started: 01/02/2001                                                 */
/*   Last Updated: 27/03/2001                                                 */
/*                                                                            */
/*         Author: Michael Bonnington                                         */
/*                                                                            */
/******************************************************************************/

/* bsp.h: Contains functions for the compilation and traversal of a BSP (Binary
 *        Space Partition) tree containing the level geometry.
 */

#ifndef BSP_H
#define BSP_H

#include <iostream.h>
#include "./opengl_includes.h"
#include "./fruity_math.h"
#include "./objloader.h"
//#include "./callbacks.h"

#define CP_BACK 0
#define CP_FRONT 1
#define CP_COINCIDENT 2
#define CP_SPLIT 3
#define TOLERANCE 0.001


/* Node structure ============================================================*/
class node
{
   public:

      node(void) {};
      ~node(void) {};

      Triangle *splitter;   //current polygon being used to partition the space

      node *front,
           *back;

      bool isLeaf,    //is this a leaf node?
           isSolid;   //is this space solid? (useful for collisions, etc.)
};


/* Level class definition ====================================================*/
class bsp : public objFile
{
   public:

      int classifyPoint(cVector3f *, Triangle *);
      int classifyPoly(Triangle *, Triangle *);
      Triangle * selectBestSplitter(Triangle *);
      void buildBSPTree(node *, Triangle *);
      void walkBSPTree(node *, cVector3f *);

      bsp(void)
      {
         BSPTreeRootNode=new node;
         buildBSPTree(BSPTreeRootNode, triangleList);
      };

      ~bsp(void) {};

      Triangle *triangleList;
      node *BSPTreeRootNode;

};




/* Node structure ============================================================*/
/*class node
{
   public:

      node(void)
      {
         sib = NULL;
      }

      void add_sibling(node *Ptr)
      {
         if(sib == NULL)
         {
            sib = Ptr;
         }
         else
         {
            sib->add_sibling(Ptr);
         }
      }

//      polygon *splitter;   //current polygon being used to partition the space

      node *sib,       //pointer to next polygon in linked list
           *infront,
           *behind;

      bool isLeaf,    //is this a leaf node?
           isSolid;   //is this space solid? (useful for collisions, etc.)
};*/

/* Polygon structure =========================================================*/
/*class polygon : public node
{
   public:

      bool triangle;
      friend void drawPoly(polygon *, float *, float *, float *);

      //constuctor...
      polygon(void) : node()
      {
         vertices = NULL;
         normals  = NULL;
         uvCoords = NULL;
      };

      // A bodge to allocate the correct amount of memory depending on the
      // polygon being a triangle or a quad...
      bool alloc(uint isTriangle)
      {
         if(vertices) delete [] vertices;
         if(normals)  delete [] normals;
         if(uvCoords) delete [] uvCoords;

         if(isTriangle == 1)
         {
            vertices = new uint[3];
            normals  = new uint[3];
            uvCoords = new uint[3];

            triangle = true;
         }
         else
         {
            vertices = new uint[4];
            normals  = new uint[4];
            uvCoords = new uint[4];

            triangle = false;
         }
         return true;
      }

      uint *vertices ,
           *normals  ,
           *uvCoords ;

      cVector3f face_normal;   //needs to be calculated and set somewhere.
                               //perhaps use fruity_plane4f.h ?

};*/


/*class BSPTRREEEEE
  {
    public :

      bool Load( char *filename )
        {
          HeadNode = LoadObjFile( filename , &m_pVertexArray , &m_pNormalArray , &m_pTexCoordArray ) ;
          if( HeadNode == NULL )
            {
              return false;
            }
          return true;
        }


    protected :


      float *m_pVertexArray;
      float *m_pNormalArray;
      float *m_pTexCoordArray;

      polygon *HeadNode;

  };*/







#endif
