/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: GLUT callbacks header file                                 */
/*        Version: 0.3 (010327)                                               */
/*                                                                            */
/*        Started: 15/11/2000                                                 */
/*   Last Updated: 27/03/2001                                                 */
/*                                                                            */
/*        Authors: Rob Bateman, Michael Bonnington                            */
/*                                                                            */
/******************************************************************************/

/* callbacks.h : Holds register callbacks function definition.
 *               Defines window structure for use with reshape and renderer.
 *               Contains data structures for game.
 */

#ifndef CALLBACKS_H
#define CALLBACKS_H

#include "./opengl_includes.h"
#include "./fruity_math.h"



/* Structs ===================================================================*/
struct window{
   int width, height, pos_x, pos_y;
   char gameModeString[16];
   bool gameMode;
};
extern struct window win;


//this is only temporary at the moment...
//some of these values will need to be initialised...
/* ===========================================================================*/
class game
{
   public:

      /* Keyboard and mouse input variables... */
      bool lmb, mmb;
      int mouse_press_x, mouse_press_y;
      GLfloat spin_x, spin_y, dolly, trans_x, trans_y;

      /* General game-related variables... */
      bool show_hud,      //Use heads-up-display stats
           show_fps,      //Show frames per second counter
           lighting_s,    //Use static lighting
           lighting_d,    //Use dynamic lighting
           use_env_map,   //Use environment mapping
           use_fog;       //Use fog

      GLfloat fog_colour[4];

      int game_mode,        //The current game mode
          option,           //The currently selected option on the title screen
          rank,             //The player's ranking on the highscore table
          time_limit,       //The maximum time limit in seconds
          cam_view,         //The current camera view
          texture_filter;   //Which texture filter to use

      float game_time,       //The current game time in seconds
            scroll_offset,   //Counter to calculate position of scrolling text
            cam_counter,     //Counter for camera moves
            msg_counter;     //Counter for displaying messages

      struct highscore_table{
         char name[32], car[32];
         float score;
      };
      struct highscore_table highscore[10];

      /* Whether to use the top-down view (for testing purposes)... */
      bool top_down_view;


      /* Member function prototypes... */
      void initData(void);   //function to initialise data
      bool loadData(void);   //function to load data
      bool saveData(void);   //function to save data

      /* Constructor... */
      game(void)
      {
         initData();
      };

      /* Destructor... */
      ~game(void)
      {
      };

};
extern game boona;

class camera
{
   public:

      cVector3f pos, aim, old_pos, old_aim;

      /* Constructor... */
      camera(float pos_x = 0, float pos_y = 0, float pos_z = 1,
             float aim_x = 0, float aim_y = 0, float aim_z = 0)
      {
         pos.set(pos_x, pos_y, pos_z);
         aim.set(aim_x, aim_y, aim_z);
      };

      /* Destructor... */
      ~camera(void)
      {
      };

};



/* Function prototypes =======================================================*/
extern void registerCallbacks(void);



#endif
