/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Car class header file                                      */
/*        Version: 0.3 (010323)                                               */
/*                                                                            */
/*        Started: 15/12/2000                                                 */
/*   Last Updated: 23/03/2001                                                 */
/*                                                                            */
/*         Author: Michael Bonnington                                         */
/*                                                                            */
/******************************************************************************/

/* car.h: The car class is derived from the model data class, enabling the
 *        drawing functions to be used within this class. Apart from this, the
 *        functionality is mainly in providing a acceptable simulation of car
 *        physics ("simulation" is not really correct, as the cars and their
 *        handling attributes are purely fantasy).
 */

#ifndef CAR_H
#define CAR_H

#include "./opengl_includes.h"
#include "./fruity_math.h"
#include "./objloader.h"

//#define PI_OVER_180 0.0174532925   //already defined somewhere else



/* Car class definition ======================================================*/
class car : public objFile
{
   public:

      /* Data to hold the car's position and direction... */
      float pos_x, pos_y, pos_z, dx, dy, dz;

      /* Member function prototypes... */
      void update(double);   //function to calculate new position
      bool drawCar(bool);    //function to draw car

      /* Constructor... */
      car(float x=0, float y=0, float z=0, float initial_speed=0) : objFile()
      {
         reset(x, y, z, initial_speed);
      };

      /* Destructor... */
      ~car(void)
      {
      };

      /* Reset... */
      inline void reset(float x=0, float y=0, float z=0, float initial_speed=0)
      {
         pos_x = x; pos_y = y; pos_z = z;
         score = 0;
         speed = initial_speed;
         angle = 0;
         heading = angle;
         dx = sin(heading*PI_OVER_180);
         dz = cos(heading*PI_OVER_180);
         left = false; right = false; reverse = false;
         brake = false; accelerator = false; handbrake = false;

         weight = 1; //max weight
         max_speed = 8;

         //vect.set(x, y, z);
      };

      /* Set position (for collisions)... */
      inline void setPos(float x=0, float y=0, float z=0,
                         float new_speed=0, float new_angle=0, float new_heading=0)
      {
         pos_x = x; pos_y = y; pos_z = z;
         speed = new_speed;
         angle = new_angle;
         heading = new_heading;
         dx = sin(heading*PI_OVER_180);
         dz = cos(heading*PI_OVER_180);
         left = false; right = false;
      };

      inline void setSpeed(float new_speed)
      {
         speed = new_speed;
      };

      /* Query functions... */
      inline float getScore(void)
      {
         return score;
      };

      inline float getSpeed(void)
      {
         return speed;
      };

      inline float getAngle(void)
      {
         return angle;
      };

      inline float getHeading(void)
      {
         return heading;
      };

      /* Player input interface functions... */
      inline void setLeft(bool state)        { left        = state; }
      inline void setRight(bool state)       { right       = state; }
      inline void setReverse(bool state)     { reverse     = state; }
      inline void setBrake(bool state)       { brake       = state; }
      inline void setAccelerator(bool state) { accelerator = state; }
      inline void setHandbrake(bool state)   { handbrake   = state; }

      inline bool getReverse(void)           { return reverse; }

   private:

      /* Player inputs affecting the car... */
      bool left, right, reverse, brake, accelerator, handbrake;

      float score, speed, angle, heading,
      weight, max_speed, acc_factor;   //modifiable car attributes

      //cVector3f vect;
};



#endif
