/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Game Engine (Based on 'Fruity Game Engine' by Rob Bateman) */
/*        Version: 0.3                                                        */
/*                                                                            */
/*        Started: 19/09/2000                                                 */
/*   Last Updated: 20/03/2001                                                 */
/*                                                                            */
/*        Authors: Michael Bonnington, Rob Bateman                            */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <fstream.h>

#include "./opengl_includes.h"
#include "./callbacks.h"



/* Main function =============================================================*/
int main(int argc, char** argv)
{
   ifstream window_opt;
   char in_str[80];

   /* Read in "WindowOptions.cfg" file... */
   window_opt.open("./config/WindowOptions.cfg", ios::in);

   if(window_opt.fail())   //set default values
   {
      win.width = 640;
      win.height = 480;
      win.pos_x = 100;
      win.pos_y = 100;
      win.gameMode = false;
   }
   else                    //load values from config file
   {
      int bit_depth, refresh_rate;

      window_opt >> in_str >> win.width;
      window_opt >> in_str >> win.height;
      window_opt >> in_str >> win.pos_x;
      window_opt >> in_str >> win.pos_y;
      window_opt >> in_str >> in_str;
      if(strcmp("TRUE", in_str) == 0)
      {
         window_opt >> in_str >> bit_depth;
         window_opt >> in_str >> refresh_rate;
         sprintf(win.gameModeString, "%dx%d:%d@%d",
                 win.width, win.height, bit_depth, refresh_rate);
         win.gameMode = true;
      }
      else win.gameMode = false;
      window_opt.close();
   }


   /* Initialise main 3D viewing window... */
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);

   /* Under Windows, the screenmode will be changed to the specified resolution,
    * bit depth and refresh rate. Under UNIX, the fullscreen mode will use the
    * same settings as the desktop.
    */
   if(win.gameMode)   //fullscreen mode
   {
      glutGameModeString(win.gameModeString);
      glutEnterGameMode();
   }
   else               //windowed mode
   {
      glutInitWindowSize(win.width, win.height);
      glutInitWindowPosition(win.pos_x, win.pos_y);
      glutCreateWindow("Boona Taxi");
   }

   registerCallbacks();

   glutMainLoop();
   return 0;
}
