/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Object loader source file                                  */
/*        Version: 0.3 (010327)                                               */
/*                                                                            */
/*        Started: 19/03/2001                                                 */
/*   Last Updated: 27/03/2001                                                 */
/*                                                                            */
/*        Authors: Rob Bateman, Michael Bonnington                            */
/*                                                                            */
/******************************************************************************/

#include "./objloader.h"


cVector3f global_light(1, 0, 0);



/*##############################################################################
###                         FUNCTION TO LOAD VERTICIES.
##############################################################################*/

bool objFile::load(char *filename)
{
   int i;

    ifstream inputFile( filename );

    if( !inputFile )
      {
      cout << "ERROR: Unable to open OBJ model file." << filename  << endl;
        return false;
      }


    else
      {

        char In_String[128];

          //
          // count data
          //
        while( !inputFile.eof() )
          {
            inputFile >> In_String;

            if( strcmp( In_String , "v"  ) == 0 ) vertexCount++;
            if( strcmp( In_String , "vn" ) == 0 ) normalCount++;
            if( strcmp( In_String , "vt" ) == 0 ) texCoordCount++;
            if( strcmp( In_String , "f"  ) == 0 ) triangleCount++;
            if( strcmp( In_String , "usemtl" ) == 0 ) MaterialCount++;

          }


          // allocate memory and set vertexCount
        vertexArray   = new Vertex  [ vertexCount   ];
        normalArray   = new Normal  [ normalCount   ];
        texCoordArray = new TexCoord[ texCoordCount ];
        triangleArray = new Triangle[ triangleCount ];
        MaterialArray = new material[ MaterialCount ];

      cout << "Material count: " << MaterialCount << endl;

        CheckMemory( vertexArray );
        CheckMemory( normalArray );
        CheckMemory( texCoordArray );
        CheckMemory( triangleArray );
        CheckMemory( MaterialArray );



          // close & re-open file stream
        inputFile.close();
        inputFile.open( filename );


          // create some temorary count values
        int vCount = 0 ,
            nCount = 0 ,
            tCount = 0 ,
            fCount = 0 ;
        MaterialCount = 0 ;

          // get data and place in array of verticies
        while( !inputFile.eof() )
          {
             inputFile >> In_String;

             if( strcmp( In_String , "v" ) == 0 )
               {
                 inputFile >> vertexArray[ vCount ].x
                           >> vertexArray[ vCount ].y
                           >> vertexArray[ vCount ].z;
                 vCount++;
               }

             if( strcmp( In_String , "vn" ) == 0 )
               {
                 inputFile >> normalArray[ nCount ].x
                           >> normalArray[ nCount ].y
                           >> normalArray[ nCount ].z;
                 nCount++;
               }

             if( strcmp( In_String , "vt" ) == 0 )
               {
                 inputFile >> texCoordArray[ tCount ].u
                           >> texCoordArray[ tCount ].v;
                 tCount++;
               }

             if( strcmp( In_String , "usemtl" ) == 0 )
               {
                 inputFile >> In_String;
                 cout << "Material # " << MaterialCount << ": " << In_String << endl;
                 MaterialCount +=1;
               }

             if( strcmp( In_String , "f" ) == 0 )
               {

                   // get the rest of the line

                 inputFile.getline( In_String , 127, '\n' );


                   // three arrays to hold indicies

                 int Verticies[3];
                 int Normals[3];
                 int TexCoords[3];   //this was 2 but I changed it to 3 - MJB


                   // remove all slashes from string

                 for( i=0; i<128; i++ )
                      if(In_String[i] == '/') In_String[i] = ' ';



                   // Scan in indicies for verticies, normals & tecoords

                 sscanf( In_String     , "%d%d%d%d%d%d%d%d%d",
                         &Verticies[0] , &TexCoords[0] , &Normals[0] ,
                         &Verticies[1] , &TexCoords[1] , &Normals[1] ,
                         &Verticies[2] , &TexCoords[2] , &Normals[2] );



                   // Need to take 1 off each index number. The *.obj index
                   // values start at 1, but array positions start at zero.

                 for( i=0; i<3; i++ )
                   {
                      Verticies[i]--;
                      Normals[i]--;
                      TexCoords[i]--;
                   }



                   // get pointers to required element of array

                 triangleArray[ fCount ].v1 = &vertexArray  [ Verticies[0] ];
                 triangleArray[ fCount ].n1 = &normalArray  [ Normals  [0] ];
                 triangleArray[ fCount ].t1 = &texCoordArray[ TexCoords[0] ];

                 triangleArray[ fCount ].v2 = &vertexArray  [ Verticies[1] ];
                 triangleArray[ fCount ].n2 = &normalArray  [ Normals  [1] ];
                 triangleArray[ fCount ].t2 = &texCoordArray[ TexCoords[1] ];

                 triangleArray[ fCount ].v3 = &vertexArray  [ Verticies[2] ];
                 triangleArray[ fCount ].n3 = &normalArray  [ Normals  [2] ];
                 triangleArray[ fCount ].t3 = &texCoordArray[ TexCoords[2] ];

                 triangleArray[ fCount ].setup();
                 triangleArray[ fCount ].calc_light();
                 triangleArray[ fCount ].mat = MaterialCount - 1 ;
                 fCount++;
               }

          }

        inputFile.close();
      }

    return true;

  }

/*##############################################################################
###                         FUNCTION TO DRAW VERTICIES.
##############################################################################*/

/*
 * Use a for loop to go through all the triangles, and draw them.
 *
*/

bool objFile::draw(void)
{
   uint i;
   int current_mat = -1;
   bool gloss;

   glDisable(GL_BLEND);

   glPushMatrix();
    glBegin(GL_TRIANGLES);

    for(i=0; i<triangleCount; i++)
    {
       if(triangleArray[i].mat != current_mat)
       {
          gloss = false;
          if(MaterialArray[triangleArray[i].mat].EnvMap && boona.use_env_map)
          {
             glEnd();
             glPopMatrix();
             glPushMatrix();
             gloss = true;
             glEnable(GL_TEXTURE_GEN_S);
             glEnable(GL_TEXTURE_GEN_T);
             glBindTexture(GL_TEXTURE_2D, envTexture);

             /* Do first rendering pass (environment map)... */
             glBegin(GL_TRIANGLES);
          }
          current_mat = triangleArray[i].mat;
       }
       if(gloss) triangleArray[i].draw();
    }

    glEnd();
   glPopMatrix();

   current_mat = -1;

   glPushMatrix();
    glBegin(GL_TRIANGLES);

    for(i=0; i<triangleCount; i++)
    {
       if(triangleArray[i].mat != current_mat)
       {
          glEnd();
          glDisable(GL_BLEND);

          /* Enable blending and blend function in env mapping is enabled... */
          if(boona.use_env_map)
          {
             if(MaterialArray[triangleArray[i].mat].EnvMap)
             {
                if(MaterialArray[triangleArray[i].mat].Gloss)
                   glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA);
                else
                   glBlendFunc(GL_ONE , GL_ONE);

                glEnable(GL_BLEND);
             }
          }

          glDisable(GL_TEXTURE_GEN_S);
          glDisable(GL_TEXTURE_GEN_T);
          glBindTexture(GL_TEXTURE_2D,
                        MaterialArray[triangleArray[i].mat].TexObjA);

          /* Do second rendering pass (surface textures)... */
          glBegin(GL_TRIANGLES);
          current_mat = triangleArray[i].mat;
       }
       triangleArray[i].draw();
    }
    glEnd();
   glPopMatrix();

   glDisable(GL_BLEND);

   return true;
}

/*##############################################################################
###                            OBJFILE DESTRUCTOR
##############################################################################*/

/*
** Delete all dynamic arrays
*/

objFile::~objFile(void)
  {
     if( vertexArray != NULL )
       {
          delete [] vertexArray;
          delete [] normalArray;
          delete [] texCoordArray;

          delete [] triangleArray;
       }
  };



/*---------------------------- MATERIAL CONSTRUCTOR --------------------------*/

material::material(void)
  {

    Gloss        = false ;
    EnvMap       = false ;

  }

//#####################################################################################
//               LOAD MATERIAL DATA FILE FUNCTION   --   MEMBER OF MODEL::
//#####################################################################################

bool objFile::LoadMD(char *filename)
{
   ifstream ifs_mat;
   char in_str[80];
   int i, j, material_number;


   ifs_mat.open(filename, ios::in);

   if(ifs_mat.fail())
   {
      cout << "ERROR: Unable to load model data file." << filename << endl;
      return false;
   }
   else
   {
      //Environment map texture...
      ifs_mat >> in_str >> in_str;
      loadTexture(in_str, &envTexture);
cout << envTexture << ": env map path: " << in_str << endl;

      for(i=0; i<(int)MaterialCount; i++)
      {
         ifs_mat >> in_str >> material_number;
         if(material_number != i)
         {
            cout << "ERROR: Invalid material number - " << material_number
                 << " (should be " << i << ")" << endl;
            return false;
         }


         //Texture unit 1 - texture path...
         ifs_mat >> in_str >> in_str;

         for(j=0; j<128; j++)
            MaterialArray[i].TexPathA[j] = in_str[j];


         //Gloss mapping Boolean...
         ifs_mat >> in_str >> in_str;

         if(strcmp(in_str, "TRUE") == 0)
            MaterialArray[i].Gloss = true;
         else
            MaterialArray[i].Gloss = false;


         //Environment mapping Boolean...
         ifs_mat >> in_str >> in_str;

         if(strcmp(in_str, "TRUE") == 0)
            MaterialArray[i].EnvMap = true;
         else
            MaterialArray[i].EnvMap = false;


         //Load textures...
         if(MaterialArray[i].TexPathA[0] != NULL)
            loadTexture(MaterialArray[i].TexPathA, &MaterialArray[i].TexObjA);

      }//end of for(int i=0; i<MaterialCount; i++)

      ifs_mat.close();
   }

   return true;
}
