/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Object Loader header file                                  */
/*        Version: 0.3 (010327)                                               */
/*                                                                            */
/*        Started: 19/03/2001                                                 */
/*   Last Updated: 27/03/2001                                                 */
/*                                                                            */
/*        Authors: Rob Bateman, Michael Bonnington                            */
/*                                                                            */
/******************************************************************************/

/* objloader.h:
 */

#ifndef OBJLOADER_H
#define OBJLOADER_H

#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include "./opengl_includes.h"
#include "./fruity_exception.h"
#include "./fruity_math.h"

#include "./callbacks.h"


extern cVector3f global_light;



class material
{
   public:

      //texture unit 1 - texture path...
      char   TexPathA[128];   //texture path
      GLuint TexObjA;         //resulting texture object

      //rendering booleans...
      bool Gloss,             //whether to render gloss map pass
           EnvMap;            //whether surface is environment mapped

      //constructor...
      material(void);

};



class Vertex : public cVector3f
{
   public:
      Vertex(void) : cVector3f() { };
};

class Normal : public cVector3f
{
   public:
      Normal(void) : cVector3f() { };
};

class TexCoord
{
   public:
      float u, v;
};


/* ===========================================================================*/

/* Triangle class: uses three pointers to vertices in the vertex array, this way
 * data can be shared rather than duplicated.
 *
 * If you wanted any 'n' sided polygon to be loaded, you can change this to use
 * arrays of pointers to the data. This way you could allocate memory to store
 * the correct number of pointers for the number of vertices.
 *
 * To work out how many faces there are you simply need to count the forward
 * slashes in the face lines when you remove them. If you count 6, you have
 * a triangle, 8 a quad, etc...
 */

class Triangle : public plane4f
{
   public:

      inline void calc_light(void)
      {
         float dot_product;

         angle1 = 0.75+(global_light*(*n1))/4;
         angle2 = 0.75+(global_light*(*n2))/4;
         angle3 = 0.75+(global_light*(*n3))/4;

         /*dot_product = global_light * (*n1);
         angle1 = divide_f(dot_product, ((*n2).length() * global_light.length()));
         angle1 = 0.5 + divide_f(acos(angle1), 2*(PI));

         dot_product = global_light * (*n2);
         angle2 = divide_f(dot_product, ((*n2).length() * global_light.length()));
         angle2 = 0.5 + divide_f(acos(angle2), 2*(PI));

         dot_product = global_light * (*n3);
         angle3 = divide_f(dot_product, ((*n2).length() * global_light.length()));
         angle3 = 0.5 + divide_f(acos(angle3), 2*(PI));*/
      }

      inline void draw(void)
      {
         if(boona.lighting_s)
         {
            //vertex 1...
            glColor3f(angle1, angle1, angle1);
            glTexCoord2f(t1->u, t1->v);
            glNormal3f(n1->x, n1->y, n1->z);
            glVertex3f(v1->x, v1->y, v1->z);

            //vertex 2...
            glColor3f(angle2, angle2, angle2);
            glTexCoord2f(t2->u, t2->v);
            glNormal3f(n2->x, n2->y, n2->z);
            glVertex3f(v2->x, v2->y, v2->z);

            //vertex 3...
            glColor3f(angle3, angle3, angle3);
            glTexCoord2f(t3->u, t3->v);
            glNormal3f(n3->x, n3->y, n3->z);
            glVertex3f(v3->x, v3->y, v3->z);
         }
         else
         {
            glColor3f(1.0, 1.0, 1.0);

            //vertex 1...
            glTexCoord2f(t1->u, t1->v);
            glNormal3f(n1->x, n1->y, n1->z);
            glVertex3f(v1->x, v1->y, v1->z);

            //vertex 2...
            glTexCoord2f(t2->u, t2->v);
            glNormal3f(n2->x, n2->y, n2->z);
            glVertex3f(v2->x, v2->y, v2->z);

            //vertex 3...
            glTexCoord2f(t3->u, t3->v);
            glNormal3f(n3->x, n3->y, n3->z);
            glVertex3f(v3->x, v3->y, v3->z);
         }
      }

      inline void setup(void)
      {
         plane4f temp(*v1, *v2, *v3);
         if( !temp.valid() ) cout << "errr ..... " << endl;
         a = temp.a;
         b = temp.b;
         c = temp.c;
         d = temp.d;
      }

      Triangle(void) : plane4f()
      {
         t1 = NULL;
         t2 = NULL;
         t3 = NULL;
         n1 = NULL;
         n2 = NULL;
         n3 = NULL;
         v1 = NULL;
         v2 = NULL;
         v3 = NULL;
         /*sibling = NULL;
         child   = NULL;*/
         nextPtr = NULL;
      };

      TexCoord *t1;
      Normal   *n1;
      Vertex   *v1;

      TexCoord *t2;
      Normal   *n2;
      Vertex   *v2;

      TexCoord *t3;
      Normal   *n3;
      Vertex   *v3;

      float angle1,
            angle2,
            angle3;

      uint mat;

      /*Triangle *sibling;
      Triangle *child;*/
      Triangle *nextPtr;

};


class objFile
{
   public:

      bool load(char *);
      bool LoadMD(char *);
      virtual bool draw(void);

      objFile(void)
      {
         vertexCount   = 0;
         normalCount   = 0;
         texCoordCount = 0;
         triangleCount = 0;
         MaterialCount = 0;
         vertexArray   = NULL;
         normalArray   = NULL;
         texCoordArray = NULL;
         triangleArray = NULL;
         MaterialArray = NULL;
      }

      ~objFile(void);

      uint      MaterialCount;   //total number of materials
      material *MaterialArray;   //array of materials


   protected:

      uint      vertexCount;     //total number of vertices
      Vertex   *vertexArray;     //array of vertices

      uint      normalCount;     //total number of normals
      Normal   *normalArray;     //array of normals

      uint      texCoordCount;   //total number of texture coordinates
      TexCoord *texCoordArray;   //array of texture coordinates

      uint      triangleCount;   //total number of triangles
      Triangle *triangleArray;   //array of triangles

      //display list number for environment map texture...
      GLuint envTexture;

};



#endif
